from .alphaquery import AlphaQuery
from .barchart import BarChart
from .etfdb import EtfDb
from .exceptions import HttpException
from .finviz import FinViz
from .stockanalysis import StockAnalysis
from .stocktwits import StockTwits
from .tipranks import TipRanks
from .yahoo import Yahoo



import re

import httpx

from .base import BaseAPI
from .exceptions import HttpException


class Yahoo(BaseAPI):
    """
    Unofficial API wrapper class for TipRanks.com.
    Unofficial means this class calls some hidden endpoints
    and provides data that official API doesn't. Also doesn't
    need an authorization.
    """

    def get_current_price_change(self):
        """
        Fetches current market price inc. pre/post market
        prices/percent/value changes. Also returns current
        market state (pre-market, open, post-market).

        Fetched stucture has following fields:

        - state (pre-market, open, post-market)
        - pre_market
            - change
                - percents
                - value
            - value
        - current_market
            - change
                - percents
                - value
            - value
        - post_market
            - change
                - percents
                - value
            - value

        Values are floats (if present) or None.
        Returned dict looks like:

        .. code-block:: python

            {
                "state": "open",
                "pre_market": {
                    "change": {
                        "percents": -1.32476,
                        "value": -1.42001
                    },
                    "value": 105.77
                },
                "current_market": {
                    "change": {
                        "percents": -1.6046284000000002,
                        "value": -1.7200012
                    },
                    "value": 105.47
                },
                "post_market": {
                    "change": {
                        "percents": 0.0,
                        "value": 0.0
                    },
                    "value": 0.0
                }
            }

        :return: Current/Pre/Post market numbers (all are floats or None).
        :rtype: dict
        """

        def get_state(data):
            if data["pre_market"]["value"]:
                return "pre-market"

            if data["post_market"]["value"]:
                return "post-market"

            return "open"

        try:
            response = self._get(
                f"https://finance.yahoo.com/quote/{self.symbol.upper()}",
                headers={
                    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/117.0"
                },
            )
        except httpx.HTTPStatusError as e:
            raise HttpException from e

        finds = re.findall(r"<fin-streamer[^>].+?>", response.text, re.DOTALL)
        need_map = {
            "regularMarketPrice": "price",
            "regularMarketChange": "change",
            "regularMarketChangePercent": "change_percents",
            "postMarketPrice": "post_market_price",
            "postMarketChange": "post_market_change",
            "postMarketChangePercent": "post_market_change_percents",
            "preMarketPrice": "pre_market_price",
            "preMarketChange": "pre_market_change",
            "preMarketChangePercent": "pre_market_change_percents",
        }
        output = {}

        if finds:
            for f in finds:
                name = re.findall(r'data-field="([a-z]+)"', f, re.IGNORECASE)

                if name[0] not in need_map.keys():
                    continue

                for key, val in need_map.items():
                    if key == name[0]:
                        value = re.findall(
                            r'value="([-0-9\.a-z]+)"', f, re.DOTALL | re.IGNORECASE
                        )

                        output[val] = value[0]
        output = {
            "pre_market": {
                "change": {
                    "percents": float(val) * 100
                    if (val := output.get("pre_market_change_percents"))
                    else None,
                    "value": float(val)
                    if (val := output.get("pre_market_change"))
                    else None,
                },
                "value": float(val)
                if (val := output.get("pre_market_price"))
                else None,
            },
            "current_market": {
                "change": {
                    "percents": float(output["change_percents"]) * 100,
                    "value": float(output["change"]),
                },
                "value": float(output["price"]),
            },
            "post_market": {
                "change": {
                    "percents": float(val) * 100
                    if (val := output.get("post_market_change_percents"))
                    else None,
                    "value": float(val)
                    if (val := output.get("post_market_change"))
                    else None,
                },
                "value": float(val)
                if (val := output.get("post_market_price"))
                else None,
            },
        }
        output["state"] = get_state(output)

        return output



import asyncio
from datetime import datetime

import httpx

from .base import BaseAPI, Data
from .exceptions import HttpException, SymbolNotFound


class TipRanks(BaseAPI):
    """
    Unofficial API wrapper class for TipRanks.com.
    Unofficial means this class calls some hidden endpoints
    and provides data that official API doesn't. Also doesn't
    need an authorization.
    """

    def get_dividends(self):
        """
        Fetches symbol dividends with following fields:

        - yield
        - amount
        - ex_date
        - payment_date
        - record_date
        - growth_since

        :return: List of dividend objects.
        :rtype: list
        """

        try:
            response = self._get(
                f"https://tr-frontend-cdn.azureedge.net/bff/prod/stock/{self.symbol.lower()}/payload.json",
            )
        except httpx.HTTPStatusError as e:
            if 404 == e.response.status_code:
                raise SymbolNotFound
            raise HttpException from e

        data = response.json()

        dividends = []

        if data["dividend"]["history"]:
            for item in data["dividend"]["history"]:
                dividends.append(
                    {
                        "yield": float(item["yield"] or 0) * 100,
                        "amount": float(item["amount"]),
                        "ex_date": datetime.strptime(
                            item["executionDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date()
                        if item["executionDate"]
                        else None,
                        "payment_date": datetime.strptime(
                            item["payDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date()
                        if item["payDate"]
                        else None,
                        "record_date": datetime.strptime(
                            item["recordDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date()
                        if item["recordDate"]
                        else None,
                        "growth_since": datetime.strptime(
                            item["growthSince"], "%Y-%m-%dT%H:%M:%S.000Z"
                        )
                        if item["growthSince"]
                        else None,
                    }
                )

        return dividends

    def get_basic_info(self):
        """
        Downloads basic info. Data are:

        - company_name
        - market
        - description
        - market_cap
        - has_dividends
        - yoy_change
        - year_low
        - year_high
        - pe_ratio
        - eps
        - similar_stocks
            - ticker
            - company_name

        :raises SymbolNotFound: In case data for the given symbol wasn't found.
        :return: Dict with data.
        :rtype: dict
        """

        async def download_basics(symbol):
            """
            Downloads basic info about symbol. Data are:

            - company_name
            - market
            - description
            - market_cap
            - similar_stocks
                - ticker
                - company_name
            - yoy_change

            :param str symbol: Symbol the data will be downloaded for.
            :return: Dict with data.
            :rtype: dict
            """

            try:
                data = await self._aget(
                    f"https://tr-frontend-cdn.azureedge.net/bff/prod/stock/{symbol.lower()}/payload.json"
                )
            except httpx.HTTPStatusError as e:
                if 404 == e.response.status_code:
                    raise SymbolNotFound
                raise HttpException from e

            json_data = Data(data.json())
            data = {
                "company_name": json_data["common"]["stock"]["fullName"] or "",
                "market": json_data["common"]["stock"]["market"] or "",
                "description": json_data["common"]["stock"]["description"] or "",
                "has_dividends": bool(
                    json_data["common"]["stock"]["dividend"]["yield"] or None
                ),
                "yoy_change": float(
                    json_data["common"]["stock"]["technical"]["momentum"][
                        "twelveMonths"
                    ]
                    or 0.0
                )
                * 100,
                "year_low": json_data["common"]["stock"]["priceRange"]["last52Week"][
                    "low"
                ]
                or 0.0,
                "year_high": json_data["common"]["stock"]["priceRange"]["last52Week"][
                    "high"
                ]
                or 0.0,
                "pe_ratio": json_data["common"]["stock"]["pe"] or 0.0,
                "similar_stocks": [],
            }

            # Find first non-null EPS.
            for earning in json_data["common"]["earnings"] or []:
                if earning["reportedEPS"]:
                    data["eps"] = earning["reportedEPS"]
                    break

            if json_data["similar"]["similar"] or []:
                for stock in json_data["similar"]["similar"]:
                    data["similar_stocks"].append(
                        {"ticker": stock["ticker"], "company_name": stock["name"]}
                    )

            return data

        async def download_additionals(symbol):
            """
            Downloads additional info. Data are:

            - year_low
            - year_high
            - pe_ratio
            - eps
            - market_cap

            :param str symbol: Symbol the data will be downloaded for.
            :return: Dict with data.
            :rtype: dict
            """

            try:
                data = await self._aget(
                    f"https://market.tipranks.com/api/details/GetRealTimeQuotes/?tickers={symbol}"
                )
            except httpx.HTTPStatusError as e:
                # Checking if symbol was even found.
                if 404 == e.response.status_code:
                    raise SymbolNotFound
                raise HttpException from e

            json_data = data.json()

            # Checking if symbol was even found.
            if 0 == len(json_data) or "currency" not in json_data[0]:
                raise SymbolNotFound

            return {
                "year_low": json_data[0]["yLow"],
                "year_high": json_data[0]["yHigh"],
                "pe_ratio": json_data[0]["pe"],
                "eps": json_data[0]["eps"],
                "market_cap": json_data[0]["marketCap"],
            }

        async def main():
            basic, additionals = await asyncio.gather(
                download_basics(self.symbol), download_additionals(self.symbol)
            )
            basic.update(additionals)

            return basic

        return asyncio.run(main())
